// (C) 2022-2024 GoodData Corporation
import { UnexpectedSdkError } from "@gooddata/sdk-ui";
import { invariant } from "ts-invariant";
import isEmpty from "lodash/isEmpty.js";
import { attributeElementsCount, filterAttributeElements, isPositiveAttributeFilter, } from "@gooddata/sdk-model";
/**
 * @internal
 */
export const ThrowMissingComponentError = (componentName, providerName) => () => {
    throw new UnexpectedSdkError(`Component: ${componentName} is missing in the ${providerName}.`);
};
/**
 * @internal
 */
export const throwMissingCallbackError = (callbackName, providerName) => (..._args) => {
    throw new UnexpectedSdkError(`Callback: ${callbackName} is missing in the ${providerName}.`);
};
/**
 * @internal
 */
export function getElementTitle(element, intl) {
    return element.formattedTitle || element.title || `(${intl.formatMessage({ id: "empty_value" })})`;
}
/**
 * @internal
 */
export function getElementTitles(elements, intl) {
    return elements.map((el) => getElementTitle(el, intl)).join(", ");
}
/**
 * @internal
 */
export function getElementKey(element) {
    return element.uri;
}
/**
 * @internal
 */
export function validateAttributeFilterProps(props) {
    const { connectToPlaceholder, filter, onApply, hiddenElements, staticElements } = props;
    invariant(!(filter && connectToPlaceholder), "It's not possible to combine 'filter' property with 'connectToPlaceholder' property. Either provide a filter, or a placeholer.");
    invariant(!(filter && !onApply), "It's not possible to use 'filter' property without 'onApply' property. Either provide 'onApply' callback or use placeholders.");
    invariant(filter || connectToPlaceholder, "No filter or placeholer provided. Provide one of the properties: 'filter', 'connectToPlaceholder'.");
    invariant(!(!isEmpty(hiddenElements) && isEmpty(staticElements)), "Hidden elements are not supported by the current backend implementation.");
}
/**
 * @internal
 */
export function isValidSingleSelectionFilter(selectionMode, filter, limitingAttributeFilters, supportsSingleSelectDependentFilters) {
    const isSingleSelect = selectionMode === "single";
    const hasEmptyParentFilters = isEmpty(limitingAttributeFilters);
    const isPositiveWithMaxOneElement = isPositiveAttributeFilter(filter) && attributeElementsCount(filterAttributeElements(filter)) < 2;
    if (isSingleSelect) {
        if (!isPositiveWithMaxOneElement) {
            console.error("Provided 'filter' or 'connectToPlaceholder' property is not compatible with given single selection mode. It needs to be positive filter with max one item selected in attribute elements");
            return false;
        }
        if (!hasEmptyParentFilters && !supportsSingleSelectDependentFilters) {
            console.error("Parent filtering can not be used together with single selection mode. Use only one of these properties at the same time.");
            return false;
        }
    }
    return true;
}
//# sourceMappingURL=utils.js.map